@@ -1,7 +1,7 @@
-/* $Id: commonrdr.h,v 1.9 2014/02/11 09:36:15 nanard Exp $ */
+/* $Id: commonrdr.h,v 1.14 2020/12/20 17:41:46 nanard Exp $ */
 /* MiniUPnP project
- * (c) 2006-2014 Thomas Bernard
- * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
+ * (c) 2006-2020 Thomas Bernard
+ * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 #ifndef COMMONRDR_H_INCLUDED
@@ -19,6 +19,11 @@ init_redirect(void);
 void
 shutdown_redirect(void);
 
+/* get_redirect_rule_count()
+ * return value : -1 for error or the number of redirection rules */
+int
+get_redirect_rule_count(const char * ifname);
+
 /* get_redirect_rule() gets internal IP and port from
  * interface, external port and protocol
  * return value :
@@ -52,5 +57,38 @@ unsigned short *
 get_portmappings_in_range(unsigned short startport, unsigned short endport,
                           int proto, unsigned int * number);
 
+/* update the port mapping internal port, description and timestamp */
+int
+update_portmapping(const char * ifname, unsigned short eport, int proto,
+                   unsigned short iport, const char * desc,
+                   unsigned int timestamp);
+
+/* update the port mapping description and timestamp */
+int
+update_portmapping_desc_timestamp(const char * ifname,
+                   unsigned short eport, int proto,
+                   const char * desc, unsigned int timestamp);
+
+#if defined(USE_NETFILTER)
+/*
+ * only provided by nftables implementation at the moment.
+ * Should be implemented for iptables too, for consistency
+ */
+
+typedef enum {
+	RDR_TABLE_NAME,
+	RDR_NAT_PREROUTING_CHAIN_NAME,
+	RDR_NAT_POSTROUTING_CHAIN_NAME,
+	RDR_FORWARD_CHAIN_NAME,
+} rdr_name_type;
+
+/*
+ * used by the config file parsing in the core
+ * to set
+ */
+
+int set_rdr_name( rdr_name_type param, const char * string );
+
 #endif
 
+#endif
