@@ -1,4 +1,4 @@
-/* $Id: natpmp.h,v 1.9 2012/09/27 15:47:15 nanard Exp $ */
+/* $Id: natpmp.h,v 1.13 2014/05/19 13:38:03 nanard Exp $ */
 /* MiniUPnP project
  * author : Thomas Bernard
  * website : http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
@@ -20,13 +20,14 @@
 
 int OpenAndConfNATPMPSockets(int * sockets);
 
-void ProcessIncomingNATPMPPacket(int s);
+/* receiveraddr is only used with IPV6 sockets */
+int ReceiveNATPMPOrPCPPacket(int s, struct sockaddr * senderaddr,
+                             socklen_t * senderaddrlen,
+                             struct sockaddr_in6 * receiveraddr,
+                             unsigned char * msg_buff, size_t msg_buff_size);
 
-#if 0
-int ScanNATPMPforExpiration(void);
-
-int CleanExpiredNATPMP(void);
-#endif
+void ProcessIncomingNATPMPPacket(int s, unsigned char * msg_buff, int len,
+                                 struct sockaddr_in * senderaddr);
 
 void SendNATPMPPublicAddressChangeNotification(int * sockets, int n_sockets);
 
