@@ -1,9 +1,9 @@
-/* $Id: upnphttp.c,v 1.106 2017/05/27 07:47:57 nanard Exp $ */
+/* $Id: upnphttp.c,v 1.109 2020/11/11 12:20:59 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * Project :  miniupnp
- * Website :  http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
+ * Website :  http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
  * Author :   Thomas Bernard
- * Copyright (c) 2005-2017 Thomas Bernard
+ * Copyright (c) 2005-2020 Thomas Bernard
  * This software is subject to the conditions detailed in the
  * LICENCE file included in this distribution.
  * */
@@ -67,9 +67,17 @@ int init_ssl(void)
 	const SSL_METHOD *method;
 	SSL_library_init();
 	SSL_load_error_strings();
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 	method = TLSv1_server_method();
+#else
+	method = TLS_server_method();
+#endif
 	if(method == NULL) {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 		syslog(LOG_ERR, "TLSv1_server_method() failed");
+#else
+		syslog(LOG_ERR, "TLS_server_method() failed");
+#endif
 		syslogsslerr();
 		return -1;
 	}
@@ -111,7 +119,11 @@ void free_ssl(void)
 		SSL_CTX_free(ssl_ctx);
 		ssl_ctx = NULL;
 	}
+#if OPENSSL_VERSION_NUMBER >= 0x10000000L && OPENSSL_VERSION_NUMBER < 0x10100000L
+	ERR_remove_thread_state(NULL);
+#elif OPENSSL_VERSION_NUMBER < 0x10000000L
 	ERR_remove_state(0);
+#endif
 	ENGINE_cleanup();
 	CONF_modules_unload(1);
 	ERR_free_strings();
@@ -762,7 +774,7 @@ ProcessHttpQuery_upnphttp(struct upnphttp * h)
 	p = h->req_buf;
 	if(!p)
 		return;
-	/* note : checking (*p != '\r') is enough to avoid runing off the
+	/* note : checking (*p != '\r') is enough to avoid running off the
 	 * end of the buffer, because h->req_buf is guaranteed to contain
 	 * the \r\n\r\n character sequence */
 	for(i = 0; i<15 && *p != ' ' && *p != '\r'; i++)
@@ -937,7 +949,19 @@ Process_upnphttp(struct upnphttp * h)
 		}
 		else if(n==0)
 		{
-			syslog(LOG_WARNING, "HTTP Connection from %s closed unexpectedly", inet_ntoa(h->clientaddr));
+#ifdef ENABLE_IPV6
+			if (h->ipv6)
+			{
+				char clientaddr_str[INET6_ADDRSTRLEN];
+				if(inet_ntop(AF_INET6, &(h->clientaddr_v6), clientaddr_str, INET6_ADDRSTRLEN) == NULL)
+					strncpy(clientaddr_str, "*inet_ntop error*", sizeof(clientaddr_str));
+				syslog(LOG_WARNING, "HTTP Connection from %s closed unexpectedly", clientaddr_str);
+			}
+			else
+#endif
+			{
+				syslog(LOG_WARNING, "HTTP Connection from %s closed unexpectedly", inet_ntoa(h->clientaddr));
+			}
 			h->state = EToDelete;
 		}
 		else
@@ -1007,7 +1031,19 @@ Process_upnphttp(struct upnphttp * h)
 		}
 		else if(n==0)
 		{
-			syslog(LOG_WARNING, "HTTP Connection from %s closed unexpectedly", inet_ntoa(h->clientaddr));
+#ifdef ENABLE_IPV6
+			if (h->ipv6)
+			{
+				char clientaddr_str[INET6_ADDRSTRLEN];
+				if(inet_ntop(AF_INET6, &(h->clientaddr_v6), clientaddr_str, INET6_ADDRSTRLEN) == NULL)
+					strncpy(clientaddr_str, "*inet_ntop error*", sizeof(clientaddr_str));
+				syslog(LOG_WARNING, "HTTP Connection from %s closed unexpectedly", clientaddr_str);
+			}
+			else
+#endif
+			{
+				syslog(LOG_WARNING, "HTTP Connection from %s closed unexpectedly", inet_ntoa(h->clientaddr));
+			}
 			h->state = EToDelete;
 		}
 		else
@@ -1216,7 +1252,7 @@ SendResp_upnphttp(struct upnphttp * h)
 			} else {
 #endif
 			if(errno == EINTR)
-				continue;	/* try again immediatly */
+				continue;	/* try again immediately */
 			if(errno == EAGAIN || errno == EWOULDBLOCK)
 			{
 				/* try again later */
@@ -1278,7 +1314,7 @@ SendRespAndClose_upnphttp(struct upnphttp * h)
 			} else {
 #endif
 			if(errno == EINTR)
-				continue;	/* try again immediatly */
+				continue;	/* try again immediately */
 			if(errno == EAGAIN || errno == EWOULDBLOCK)
 			{
 				/* try again later */
