@@ -1,8 +1,8 @@
-/* $Id: upnpsoap.c,v 1.151 2018/03/13 10:32:53 nanard Exp $ */
+/* $Id: upnpsoap.c,v 1.160 2020/10/30 21:37:33 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
- * (c) 2006-2018 Thomas Bernard
+ * (c) 2006-2020 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -27,6 +27,7 @@
 #include "upnpsoap.h"
 #include "upnpreplyparse.h"
 #include "upnpredirect.h"
+#include "upnppermissions.h"
 #include "upnppinhole.h"
 #include "getifaddr.h"
 #include "getifstats.h"
@@ -337,11 +338,18 @@ GetExternalIPAddress(struct upnphttp * h, const char * action, const char * ns)
 		strncpy(ext_ip_addr, use_ext_ip_addr, INET_ADDRSTRLEN);
 		ext_ip_addr[INET_ADDRSTRLEN - 1] = '\0';
 	}
-	else if(getifaddr(ext_if_name, ext_ip_addr, INET_ADDRSTRLEN, NULL, NULL) < 0)
+	else
 	{
-		syslog(LOG_ERR, "Failed to get ip address for interface %s",
-			ext_if_name);
-		strncpy(ext_ip_addr, "0.0.0.0", INET_ADDRSTRLEN);
+		struct in_addr addr;
+		if(getifaddr(ext_if_name, ext_ip_addr, INET_ADDRSTRLEN, &addr, NULL) < 0)
+		{
+			syslog(LOG_ERR, "Failed to get ip address for interface %s",
+				ext_if_name);
+			strncpy(ext_ip_addr, "0.0.0.0", INET_ADDRSTRLEN);
+		} else if (addr_is_reserved(&addr)) {
+			syslog(LOG_NOTICE, "private/reserved address %s is not suitable for external IP", ext_ip_addr);
+			strncpy(ext_ip_addr, "0.0.0.0", INET_ADDRSTRLEN);
+		}
 	}
 #else
 	struct lan_addr_s * lan_addr;
@@ -356,6 +364,11 @@ GetExternalIPAddress(struct upnphttp * h, const char * action, const char * ns)
 		}
 	}
 #endif
+	if (strcmp(ext_ip_addr, "0.0.0.0") == 0)
+	{
+		SoapError(h, 501, "Action Failed");
+		return;
+	}
 	bodylen = snprintf(body, sizeof(body), resp,
 	              action, ns, /*SERVICE_TYPE_WANIPC,*/
 				  ext_ip_addr, action);
@@ -410,7 +423,7 @@ AddPortMapping(struct upnphttp * h, const char * action, const char * ns)
 	r_host = GetValueFromNameValueList(&data, "NewRemoteHost");
 #ifndef SUPPORT_REMOTEHOST
 #ifdef UPNP_STRICT
-	if (r_host && (strlen(r_host) > 0) && (0 != strcmp(r_host, "*")))
+	if (r_host && (r_host[0] != '\0') && (0 != strcmp(r_host, "*")))
 	{
 		ClearNameValueList(&data);
 		SoapError(h, 726, "RemoteHostOnlySupportsWildcard");
@@ -513,6 +526,7 @@ AddPortMapping(struct upnphttp * h, const char * action, const char * ns)
 	/* possible error codes for AddPortMapping :
 	 * 402 - Invalid Args
 	 * 501 - Action Failed
+	 * 606 - Action not authorized (added in IGD v2)
 	 * 715 - Wildcard not permited in SrcAddr
 	 * 716 - Wildcard not permited in ExtPort
 	 * 718 - ConflictInMappingEntry
@@ -529,7 +543,8 @@ AddPortMapping(struct upnphttp * h, const char * action, const char * ns)
      * 728 - NoPortMapsAvailable
              There are not enough free ports available to complete the mapping
              (added in IGD v2)
-	 * 729 - ConflictWithOtherMechanisms (added in IGD v2) */
+	 * 729 - ConflictWithOtherMechanisms (added in IGD v2)
+	 * 732 - WildCardNotPermittedInIntPort (added in IGD v2) */
 	switch(r)
 	{
 	case 0:	/* success */
@@ -542,8 +557,12 @@ AddPortMapping(struct upnphttp * h, const char * action, const char * ns)
 		SoapError(h, 729, "ConflictWithOtherMechanisms");
 		break;
 #endif /* IGD_V2 */
-	case -2:	/* already redirected */
 	case -3:	/* not permitted */
+#ifdef IGD_V2
+		SoapError(h, 606, "Action not authorized");
+		break;
+#endif /* IGD_V2 */
+	case -2:	/* already redirected */
 		SoapError(h, 718, "ConflictInMappingEntry");
 		break;
 	default:
@@ -590,23 +609,26 @@ AddAnyPortMapping(struct upnphttp * h, const char * action, const char * ns)
 	if(leaseduration == 0)
 		leaseduration = 604800;
 
-	if (!int_ip || !ext_port || !int_port)
+	if (!int_ip || !ext_port || !int_port || !protocol)
 	{
 		ClearNameValueList(&data);
 		SoapError(h, 402, "Invalid Args");
 		return;
 	}
 
-	eport = (unsigned short)atoi(ext_port);
+	eport = (0 == strcmp(ext_port, "*")) ? 0 : (unsigned short)atoi(ext_port);
+	if (eport == 0) {
+		eport = 1024 + ((random() & 0x7ffffffL) % (65536-1024));
+	}
 	iport = (unsigned short)atoi(int_port);
-	if(iport == 0 || !is_numeric(ext_port)) {
+	if(iport == 0 || (!is_numeric(ext_port) && 0 != strcmp(ext_port, "*"))) {
 		ClearNameValueList(&data);
 		SoapError(h, 402, "Invalid Args");
 		return;
 	}
 #ifndef SUPPORT_REMOTEHOST
 #ifdef UPNP_STRICT
-	if (r_host && (strlen(r_host) > 0) && (0 != strcmp(r_host, "*")))
+	if (r_host && (r_host[0] != '\0') && (0 != strcmp(r_host, "*")))
 	{
 		ClearNameValueList(&data);
 		SoapError(h, 726, "RemoteHostOnlySupportsWildcard");
@@ -651,14 +673,42 @@ AddAnyPortMapping(struct upnphttp * h, const char * action, const char * ns)
 		}
 	}
 
-	/* TODO : accept a different external port
-	 * have some smart strategy to choose the port */
-	for(;;) {
-		r = upnp_redirect(r_host, eport, int_ip, iport, protocol, desc, leaseduration);
-		if(r==-2 && eport < 65535) {
-			eport++;
-		} else {
-			break;
+	/* first try the port asked in request, then
+	 * try +1, -1, +2, -2, etc. */
+	r = upnp_redirect(r_host, eport, int_ip, iport, protocol, desc, leaseduration);
+	if (r != 0 && r != -1) {
+		unsigned short eport_below, eport_above;
+		struct in_addr address;
+		uint32_t allowed_eports[65536 / 32];
+
+		if(inet_aton(int_ip, &address) <= 0) {
+			syslog(LOG_ERR, "inet_aton(%s) FAILED", int_ip);
+		}
+		get_permitted_ext_ports(allowed_eports, upnppermlist, num_upnpperm,
+		                        address.s_addr, iport);
+		eport_above = eport_below = eport;
+		for(;;) {
+			/* loop invariant
+			 * eport is equal to either eport_below or eport_above (or both) */
+			if (eport_below <= 1 && eport_above == 65535) {
+				/* all possible ports tried */
+				r = 1;
+				break;
+			}
+			if (eport_above == 65535 || (eport > eport_below && eport_below > 1)) {
+				eport = --eport_below;
+			} else {
+				eport = ++eport_above;
+			}
+			if (!(allowed_eports[eport / 32] & ((uint32_t)1U << (eport % 32))))
+				continue;	/* not allowed */
+			r = upnp_redirect(r_host, eport, int_ip, iport, protocol, desc, leaseduration);
+			if (r == 0 || r == -1) {
+				/* OK or failure : Stop */
+				break;
+			}
+			/* r : -2 / -4 already redirected or -3 permission check failed :
+			 * continue */
 		}
 	}
 
@@ -666,6 +716,9 @@ AddAnyPortMapping(struct upnphttp * h, const char * action, const char * ns)
 
 	switch(r)
 	{
+	case 1:	/* exhausted possible mappings */
+		SoapError(h, 728, "NoPortMapsAvailable");
+		break;
 	case 0:	/* success */
 		bodylen = snprintf(body, sizeof(body), resp,
 		              action, ns, /*SERVICE_TYPE_WANIPC,*/
@@ -724,7 +777,7 @@ GetSpecificPortMappingEntry(struct upnphttp * h, const char * action, const char
 	}
 #ifndef SUPPORT_REMOTEHOST
 #ifdef UPNP_STRICT
-	if (r_host && (strlen(r_host) > 0) && (0 != strcmp(r_host, "*")))
+	if (r_host && (r_host[0] != '\0') && (0 != strcmp(r_host, "*")))
 	{
 		ClearNameValueList(&data);
 		SoapError(h, 726, "RemoteHostOnlySupportsWildcard");
@@ -757,10 +810,10 @@ GetSpecificPortMappingEntry(struct upnphttp * h, const char * action, const char
 	}
 	else
 	{
-		syslog(LOG_INFO, "%s: rhost='%s' %s %s found => %s:%u desc='%s'",
+		syslog(LOG_INFO, "%s: rhost='%s' %s %s found => %s:%u desc='%s' duration=%u",
 		       action,
 		       r_host ? r_host : "NULL", ext_port, protocol, int_ip,
-		       (unsigned int)iport, desc);
+		       (unsigned int)iport, desc, leaseduration);
 		bodylen = snprintf(body, sizeof(body), resp,
 				action, ns/*SERVICE_TYPE_WANIPC*/,
 				(unsigned int)iport, int_ip, desc, leaseduration,
@@ -813,7 +866,7 @@ DeletePortMapping(struct upnphttp * h, const char * action, const char * ns)
 	}
 #ifndef SUPPORT_REMOTEHOST
 #ifdef UPNP_STRICT
-	if (r_host && (strlen(r_host) > 0) && (0 != strcmp(r_host, "*")))
+	if (r_host && (r_host[0] != '\0') && (0 != strcmp(r_host, "*")))
 	{
 		ClearNameValueList(&data);
 		SoapError(h, 726, "RemoteHostOnlySupportsWildcard");
@@ -1519,23 +1572,35 @@ PinholeVerification(struct upnphttp * h, char * int_ip, unsigned short int_port)
 	if (inet_pton(AF_INET6, int_ip, &result_ip) <= 0)
 	{
 		n = getaddrinfo(int_ip, NULL, &hints, &ai);
-		if(!n && ai->ai_family == AF_INET6)
+		if (n == 0)
 		{
+			int found = 0;
 			for(p = ai; p; p = p->ai_next)
 			{
-				inet_ntop(AF_INET6, (struct in6_addr *) p, int_ip, sizeof(struct in6_addr));
-				result_ip = *((struct in6_addr *) p);
-				/* TODO : deal with more than one ip per hostname */
-				break;
+				if(p->ai_family == AF_INET6)
+				{
+					inet_ntop(AF_INET6, (struct in6_addr *) p, int_ip, sizeof(struct in6_addr));
+					result_ip = *((struct in6_addr *) p);
+					found = 1;
+					/* TODO : deal with more than one ip per hostname */
+					break;
+				}
+			}
+			freeaddrinfo(ai);
+			if (!found)
+			{
+				syslog(LOG_NOTICE, "No IPv6 address for hostname '%s'", int_ip);
+				SoapError(h, 402, "Invalid Args");
+				return -1;
 			}
 		}
 		else
 		{
-			syslog(LOG_ERR, "Failed to convert hostname '%s' to ip address", int_ip);
+			syslog(LOG_WARNING, "Failed to convert hostname '%s' to IP address : %s",
+			       int_ip, gai_strerror(n));
 			SoapError(h, 402, "Invalid Args");
 			return -1;
 		}
-        freeaddrinfo(p);
 	}
 
 	if(inet_ntop(AF_INET6, &(h->clientaddr_v6), senderAddr, INET6_ADDRSTRLEN) == NULL)
@@ -1618,7 +1683,7 @@ AddPinhole(struct upnphttp * h, const char * action, const char * ns)
 	 * - InternalClient value equals to the control point's IP address.
 	 * It is REQUIRED that InternalClient cannot be one of IPv6
 	 * addresses used by the gateway. */
-	if(!int_ip || 0 == strlen(int_ip) || 0 == strcmp(int_ip, "*"))
+	if(!int_ip || int_ip[0] == '\0' || 0 == strcmp(int_ip, "*"))
 	{
 		SoapError(h, 708, "WildCardNotPermittedInSrcIP");
 		goto clear_and_exit;
@@ -1842,6 +1907,13 @@ GetOutboundPinholeTimeout(struct upnphttp * h, const char * action, const char *
 	rem_port = GetValueFromNameValueList(&data, "RemotePort");
 	protocol = GetValueFromNameValueList(&data, "Protocol");
 
+	if (!int_port || !rem_port || !protocol)
+	{
+		ClearNameValueList(&data);
+		SoapError(h, 402, "Invalid Args");
+		return;
+	}
+
 	rport = (unsigned short)atoi(rem_port);
 	iport = (unsigned short)atoi(int_port);
 	/*proto = atoi(protocol);*/
