@@ -1,12 +1,12 @@
-/* $Id: upnpurns.h,v 1.2 2012/03/05 20:36:19 nanard Exp $ */
+/* $Id: upnpurns.h,v 1.3 2012/09/27 15:46:18 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006-2011 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
-#ifndef __UPNPURNS_H__
-#define __UPNPURNS_H__
+#ifndef UPNPURNS_H_INCLUDED
+#define UPNPURNS_H_INCLUDED
 
 #include "config.h"
 
